"""
Module which does the acutal work on the database.

SnortDB To SoapSy (SnDB2Soapsy)

@author: Michael Pilgermann
@contact: mailto:mpilgerm@glam.ac.uk
@license: GPL (General Public License)
"""

import time
import config
import pg
import dataengine_tools
import dbconnector


class DatabaseAction:

    def __init__(self):
        """
        Loads settings from the config file.
        
        Use L{startup} to make this module really running!
        """
        self._dbhost = config.DB_HOST
        self._dbport = config.DB_PORT 
        self._dbname = config.DB_DATABASENAME 
        self._dbuser = config.DB_USERNAME 
        self._dbpassword = config.DB_PASSWORD 
        self._dboptions = None
        self._dbtty = None
        
        self._interval = config.DB_POLL_INTERVAL
    
    def startup(self):
        """
        Establishes the database connection regarding to settings in the config file and
        keeps us running forever.
        """
        self._dictCreator = dataengine_tools.getPreXMLDictCreator()
        self._DBConnector = dbconnector.getDBConnector()
        self._DBConnector.connect()
        
        lastestEventOid = '0'
        try:
            filename = config.EVENT_STATUS_LOCATION
            file = open(filename, 'r')
            lastestEventOid = file.readline()
            file.close()
        except Exception, msg:
            pass
        
        lastestEventOid = int(lastestEventOid)
        self._connection = pg.connect(self._dbname, self._dbhost, self._dbport, self._dboptions, self._dbtty, self._dbuser, self._dbpassword)
        while 1:
            lastestEventOid = self._triggerNow(lastestEventOid)
            file = open(filename, 'w')
            file.write(str(lastestEventOid) + "\n")
            file.close()
            time.sleep(self._interval)
        
        
    def _getEventsFromDbOverOid(self, min_oid):
        query = """select oid, sid, cid, signature, timestamp from event where oid > '""" + str(min_oid) + """'"""
        result = self._connection.query(query)
        list = result.getresult()
        returnList = []
        for item in list:
            newItem = {}
            newItem['oid'] = item[0]
            newItem['sid'] = item[1]
            newItem['cid'] = item[2]
            newItem['signature'] = item[3]
            newItem['timestamp'] = item[4]
            returnList.append(newItem)
            if int(item[0]) > min_oid:
                min_oid = int(item[0])
        return returnList, min_oid
        
    def _getSensorInformation(self, sid):
        query = """select sid, hostname from sensor where sid = '""" + str(sid) + """'"""
        result = self._connection.query(query)
        list = result.getresult()
        item = list[0]
        newSensor = {}
        newSensor['name'] = item[1]
        return newSensor
        
    def _createObserverFromSensor(self, sensor):
        computer =self._dictCreator.createNewComputerEntry(sensor['name'], None, None, None, None, [])
        agent = self._dictCreator.createNewAgentEntry(config.SOAPSY_OBSERVER_AGENT_NAME, [computer], '2')
        observer = self._dictCreator.createNewObserverEntry(config.SOAPSY_OBSERVER_NAME,[agent])
        return observer
        
    def _createLocalReporter(self):
        computer = self._dictCreator.createNewComputerEntry(config.SOAPSY_LOCAL_NAME, 
            config.SOAPSY_LOCAL_OS, config.SOAPSY_LOCAL_IP, config.SOAPSY_LOCAL_MAC, config.SOAPSY_LOCAL_DOMAIN, [])
        agent = self._dictCreator.createNewAgentEntry(config.SOAPSY_LOCAL_AGENT_NAME, [computer], '2')
        reporter = self._dictCreator.createNewReporterEntry(config.SOAPSY_LOCAL_REPORTER_NAME,[agent])
        return reporter
        
    def _convertIPDecToStr(self, ipDec):
        ipStr = ''
        for i in range(0,3):
            ipStr += str(ipDec % 256) + "."
            ipDec = ipDec / 256
        ipStr += str(ipDec)
        return ipStr
        
    def _getSrcAndDestIP(self, sid, cid):
        query = """select ip_src, ip_dst from iphdr where sid = '""" + str(sid) + """' and cid = '""" + str(cid) + """'"""
        result = self._connection.query(query)
        list = result.getresult()
        if len(list) == 0:
            return None, None
        item = list[0]
        srcIP = item[0]
        dstIP = item[1]
        return self._convertIPDecToStr(srcIP), self._convertIPDecToStr(dstIP)
        
    def _createSourceFromIP(self, ip):
        computer =self._dictCreator.createNewComputerEntry(ip, None, ip, None, None, [])
        agent = self._dictCreator.createNewAgentEntry(config.SOAPSY_SOURCE_AGENT_NAME, [computer], '2')
        source = self._dictCreator.createNewSourceEntry(config.SOAPSY_SOURCE_NAME,[agent])
        return source

    def _createDestinationFromIP(self, ip):
        computer =self._dictCreator.createNewComputerEntry(ip, None, ip, None, None, [])
        agent = self._dictCreator.createNewAgentEntry(config.SOAPSY_DESTINATION_AGENT_NAME, [computer], '2')
        destination = self._dictCreator.createNewDestinationEntry(config.SOAPSY_DESTINATION_NAME,[agent])
        return destination
        
        
    def _triggerNow(self, lastestEventOid):
        dictEvents, lastestEventOid = self._getEventsFromDbOverOid(lastestEventOid)
        for event in dictEvents:
            sensor = self._getSensorInformation(event['sid'])
            observer = self._createObserverFromSensor(sensor)
            reporter = self._createLocalReporter()
            srcIP, dstIP = self._getSrcAndDestIP(event['sid'], event['cid'])
            source = self._createSourceFromIP(srcIP)
            destination = self._createDestinationFromIP(dstIP)
            eventType = self._dictCreator.createNewEventTypeEntry(config.SOAPSY_EVENT_TYPE_NAME)
            
            eventEntry = self._dictCreator.createNewEventEntry(event['timestamp'], [observer, reporter, source, destination, eventType])
            primKey = self._DBConnector.insertEvent(eventEntry)
            
            print "Inserted event into SoapSy with key %s" %(primKey)
##            import ioids.support.dictviewer
##            ioids.support.dictviewer.showNow(eventEntry)
            
        return lastestEventOid
        
    def shutdown(self):
        self._connection.close()
        self._DBConnector.disconnect()
        
